/*
 * licy.js v0.5.0 http://mantoni.github.com/licy.js
 *
 * @generator nomo.js v0.1.5
 */
(function (modules) {
  'use strict';
  var instances = {};
  var delegate = window.require;

  function require(name) {
    if (!instances.hasOwnProperty(name)) {
      var factory = modules[name];
      if (!factory) {
        if (delegate) {
          return instances[name] = { exports: delegate(name) };
        }
        throw new Error('Unknown module: ' + name);
      }
      var exports = {};
      instances[name] = { exports: exports };
      factory(instances[name], exports, require);
    }
    return instances[name].exports;
  }

  require.define = function (name, factory) {
    if (modules[name]) {
      throw new Error('Module "' + name + '" already defined.');
    }
    modules[name] = factory;
  };

  window.licy = require('lib/licy');

}({
  'lib/licy': function (module, exports, require) {
    /*
     * licy.js
     *
     * Copyright (c) 2012-2013 Maximilian Antoni <mail@maxantoni.de>
     *
     * @license MIT
     */
    'use strict';
    function typeOf(value) {
      if (value === null) {
        return 'null';
      }
      var type = Object.prototype.toString.call(value);
      return type.substring(8, type.length - 1).toLowerCase();
    }

    function assertType(value, expectation, name) {
      var type = typeOf(value);
      if (type !== expectation) {
        throw new TypeError('Expected ' + name + ' to be ' + expectation +
          ', but it was ' + type);
      }
    }

    function err(name, message) {
      throw new Error('Plugin "' + name + '" ' + message);
    }


    var hub  = require('node_modules/hubjs/lib/hub');
    var licy = hub();

    var PLUGIN_NOT_ENOUGH_ARGS_ERRORS = [
      'No arguments given.',
      'No start function given.'
    ];

    function emit(name, event, view) {
      licy.emit('licy.' + name + '.' + event, view);
    }

    var started = false;
    var destroyOrder = [];

    licy.plugin = function (name, start) {
      if (arguments.length < 2) {
        throw new TypeError(PLUGIN_NOT_ENOUGH_ARGS_ERRORS[arguments.length]);
      }
      assertType(name, 'string', 'name');
      assertType(start, 'function', 'start');
      if (started) {
        throw new Error('Cannot register plugins after start');
      }
      var startEvent   = name + '.start';
      var destroyEvent = name + '.destroy';

      licy.on('licy.' + startEvent, function () {
        licy.emit(startEvent, this.callback());
      });
      licy.on('licy.' + destroyEvent, function () {
        licy.emit(destroyEvent, this.callback());
      });

      var queue = [];
      var queueListener = function () {
        if (this.event !== startEvent) {
          queue.push(this.callback());
        }
      };
      function queueEvents() {
        licy.before(name + '.**', queueListener);
      }
      var autoStarted;
      function register() {
        autoStarted = false;
        var view;
        queueEvents();
        licy.onceBefore(name + '.**', function () {
          if (!view && this.event !== startEvent && this.event !== destroyEvent) {
            autoStarted = true;
            /*
             * Pass an empty function to prevent an error from being thrown.
             * Errors are passed to the emit callback or thrown there, so we don't
             * have to handle it here.
             */
            licy.emit(startEvent, function () {});
          }
        });
        licy.before(startEvent, function () {
          if (view) {
            if (autoStarted) {
              autoStarted = false;
              return;
            }
            err(name, 'already started');
          }
          started = true;
          view = licy.view(name);
          emit(name, 'starting', view);
          if (start.length < 2) {
            start.call(null, view);
            return;
          }
          var callback = this.callback(5000);
          try {
            start.call(null, view, callback);
          } catch (e) {
            callback(e);
          }
        });
        licy.onceAfter(startEvent, function (err) {
          if (err) {
            var message = err.message;
            err.message = 'Failed to start plugin "' + name + '"';
            if (message) {
              err.message += ': ' + message;
            }
          } else {
            emit(name, 'started', view);
          }
          licy.un(name + '.**', queueListener);
          var i, l = queue.length;
          for (i = 0; i < l; i++) {
            queue[i](err);
          }
          queue.length = 0; // TODO write test
        });

        licy.onceBefore(destroyEvent, function () {
          emit(name, 'destroying', view);
          queueEvents();
        });
        licy.onceAfter(destroyEvent, function () {
          view.removeAllListeners();
          register();
          emit(name, 'destroyed', view);
        });
      }
      licy.onceAfter(startEvent, function (err) {
        if (!err) {
          destroyOrder[autoStarted ? 'push' : 'unshift'](name);
        }
      });
      register();
    };

    licy.plugins = function (plugins) {
      if (!arguments.length) {
        throw new TypeError(PLUGIN_NOT_ENOUGH_ARGS_ERRORS[0]);
      }
      assertType(plugins, 'object', 'plugins');
      var key;
      for (key in plugins) {
        if (plugins.hasOwnProperty(key)) {
          licy.plugin(key, plugins[key]);
        }
      }
    };

    function emits(event) {
      return function (name, callback) {
        assertType(name, 'string', 'name');
        licy.emit('licy.' + name + '.' + event, callback);
      };
    }

    function destroyNext(listener, callback) {
      if (destroyOrder.length) {
        licy.emit(destroyOrder.shift() + '.destroy', listener(function () {
          destroyNext(listener, callback);
        }));
      } else {
        listener.then(callback || function () {});
      }
    }

    licy.start   = emits('start');
    licy.destroy = emits('destroy');
    licy.restart = function (name, callback) {
      licy.destroy(name, function () {
        licy.emit('licy.' + name + '.start', callback);
      });
    };
    licy.startAll = function (callback) {
      started = true;
      licy.emit('licy.**.start', callback);
    };
    licy.destroyAll = function (callback) {
      destroyNext(hub.listen(), callback);
    };
    licy.reset = function () {
      licy.removeAllListeners();
      destroyOrder = [];
      started = false;
    };

    licy.listen  = hub.listen;
    licy.View    = hub.View;
    licy.options = hub.options;
    licy.Options = hub.Options;

    module.exports = licy;

  },
  'node_modules/hubjs/lib/hub': function (module, exports, require) {
    /*
     * hub.js
     *
     * Copyright (c) 2012-2013 Maximilian Antoni <mail@maxantoni.de>
     *
     * @license MIT
     */
    'use strict';
    var listen = require('node_modules/hubjs/node_modules/listen/lib/listen');


    function emit(listeners, scope, args, values, callback) {
      if (listeners) {
        var listener = listen(values),
          fns        = listeners.slice(),
          argsLength = args.length,
          l          = listeners.length,
          i;
        scope.callback = listener;

        for (i = 0; i < l; i++) {
          var fn    = fns[i];
          var async = fn.length > argsLength;
          if (async) {
            args[fn.length - 1] = listener();
          }
          var value;
          try {
            value = fn.apply(scope, args);
          } catch (e) {
            if (async) {
              args[fn.length - 1](e);
            } else {
              listener.err(e);
            }
          }
          if (value !== undefined) {
            if (async) {
              args.pop()(null, value);
            } else {
              listener.push(value);
            }
          } else if (async) {
            args.pop();
          }
        }

        listener.then(function (err, values) {
          callback.call(scope, err, values);
        });
      } else {
        callback.call(scope, null, values);
      }
    }


    function eventRE(event) {
      return new RegExp('^' + event.replace(/\./g, '\\.').
        replace(/\*\*/g, '[%\\.]+').replace(/\*/g, '[%]+').
        replace(/%/g, 'a-zA-Z_0-9\\-\\:\\*') + '$');
    }


    function call(fns, err) {
      var i, l = fns.length, thrown;
      for (i = 0; i < l; i++) {
        try {
          fns[i](err);
        } catch (e) {
          thrown = e;
        }
      }
      if (thrown) {
        throw thrown;
      }
    }


    function done(scope, callback, err, result) {
      if (callback) {
        callback.call(scope, err, result);
      } else if (err) {
        var fns, dotIndex = scope.event.lastIndexOf('.');
        if (dotIndex !== -1) {
          var errorEvent = scope.event.substring(0, dotIndex) + '.error';
          fns = scope.hub.listeners(errorEvent);
          if (fns.length) {
            call(fns, err);
            return;
          }
        }
        fns = scope.hub.listeners('error');
        if (fns.length) {
          call(fns, err);
          return;
        }
        throw err;
      }
    }


    function indexOf(array, object) {
      var i, l = array.length;
      for (i = 0; i < l; i++) {
        if (array[i] === object) {
          return i;
        }
      }
      return -1;
    }


    function remove(array, object) {
      var i = indexOf(array, object);
      if (i !== -1) {
        array.splice(i, 1);
      }
    }


    function typeOf(value) {
      if (value === null) {
        return 'null';
      }
      var type = Object.prototype.toString.call(value);
      return type.substring(8, type.length - 1).toLowerCase();
    }


    function assertType(value, expectation, name) {
      var type = typeOf(value);
      if (type !== expectation) {
        throw new TypeError('Expected ' + name + ' to be ' + expectation +
          ', but it was ' + type);
      }
    }


    function noop() {}


    var NOT_ENOUGH_ARGS_ERRORS = [
      'No arguments given.',
      'No listener function given.'
    ];

    function multiple(delegate) {
      return function (object, fn) {
        /*jslint forin: true*/
        if (typeOf(object) === 'object') {
          var event, listener;
          for (event in object) {
            listener = object[event];
            if (typeof listener === 'function') {
              delegate.call(this, event, listener);
            }
          }
        } else {
          if (arguments.length < 2) {
            throw new TypeError(NOT_ENOUGH_ARGS_ERRORS[arguments.length]);
          }
          assertType(object, 'string', 'event');
          assertType(fn, 'function', 'listener');
          delegate.call(this, object, fn);
        }
      };
    }


    function View(hub, namespace) {
      this.hub        = hub;
      this.namespace  = namespace;
    }
    View.prototype = {
      toString: function () {
        return '[object hub.View(' + this.namespace + ')]';
      },
      emit: function (event) {
        if (arguments.length === 0) {
          throw new TypeError(NOT_ENOUGH_ARGS_ERRORS[0]);
        }
        assertType(event, 'string', 'event');
        var args = Array.prototype.slice.call(arguments, 1);
        args.unshift(this.namespace + '.' + event);
        this.hub.emit.apply(this.hub, args);
      },
      removeAllListeners: function (event) {
        if (arguments.length) {
          assertType(event, 'string', 'event');
          this.hub.removeAllListeners(this.namespace + '.' + event);
        } else {
          this.hub.removeAllMatching(this.namespace + '.**');
        }
      }
    };
    var viewProto = View.prototype;
    function viewDelegateOneArg(method) {
      viewProto[method] = function (event) {
        assertType(event, 'string', 'event');
        return this.hub[method](this.namespace + '.' + event);
      };
    }
    function viewDelegateTwoArgs(method) {
      viewProto[method] = multiple(function (event, callback) {
        this.hub[method](this.namespace + '.' + event, callback);
      });
    }
    viewDelegateOneArg('view');
    viewDelegateOneArg('removeAllMatching');
    viewDelegateTwoArgs('on');
    viewDelegateTwoArgs('un');
    viewDelegateTwoArgs('before');
    viewDelegateTwoArgs('after');
    viewDelegateTwoArgs('once');
    viewDelegateTwoArgs('onceBefore');
    viewDelegateTwoArgs('onceAfter');
    viewProto.addListener = viewProto.on;
    viewProto.removeListener = viewProto.un;


    function clearEntry(entry) {
      entry.before.length = 0;
      entry.on.length     = 0;
      entry.after.length  = 0;
    }


    function entryListeners(entry) {
      return entry.before.concat(entry.on).concat(entry.after);
    }


    function createPrototype(proto) {
      function F() {}
      F.prototype = proto;
      return new F();
    }

    function returnsTrue() {
      return true;
    }

    function returnsFalse() {
      return false;
    }

    function Options(config) {
      if (config.hasOwnProperty('allResults')) {
        assertType(config.allResults, 'boolean', 'allResults');
      }
      this.allResults = config.allResults ? returnsTrue : returnsFalse;
    }
    Options.prototype.toString = function () {
      return '[object hub.Options]';
    };
    var DEFAULT_OPTIONS = new Options({});


    function result(scope, values) {
      return values ? (scope.options.allResults() ?
          values : values[values.length - 1]) : null;
    }


    var arrayPush = Array.prototype.push;


    function create() {
      var listeners = {};
      var matchers  = [];

      function listener(event) {
        var entry = listeners[event];
        if (entry) {
          return entry;
        }
        return (listeners[event] = { before : [], on : [], after : [] });
      }

      function matcher(event) {
        var i, l = matchers.length, entry;
        for (i = 0; i < l; i++) {
          entry = matchers[i];
          if (entry.event === event) {
            return entry;
          }
        }
        var cmp = event.replace(/[a-zA-Z_0-9\-\:]+/g, '0')
                        .replace(/\./g, '1')
                        .replace(/\*/g, '2');
        entry = {
          event   : event,
          cmp     : cmp,
          re      : eventRE(event),
          before  : [],
          on      : [],
          after   : []
        };
        for (i = 0, l = matchers.length; i < l; i++) {
          if (cmp > matchers[i].cmp) {
            matchers.splice(i, 0, entry);
            return entry;
          }
        }
        matchers.push(entry);
        return entry;
      }

      function findListeners(type, event, re) {
        var entry;
        if (re) {
          var evt, entries;
          for (evt in listeners) {
            if (listeners.hasOwnProperty(evt) && re.test(evt)) {
              entry   = listeners[evt][type];
              entries = entries ? entries.concat(entry) : entry;
            }
          }
          return entries;
        }
        entry = listeners[event];
        return entry ? entry[type] : null;
      }

      function findMatchers(type, event, re) {
        var i, l = matchers.length, entries;
        for (i = 0; i < l; i++) {
          var matcher = matchers[i];
          if (matcher.re.test(event) || (re && re.test(matcher.event))) {
            entries = entries ? entries.concat(matcher[type]) : matcher[type];
          }
        }
        return entries;
      }

      function register(type) {
        return multiple(function (event, fn) {
          this.emit('newListener', event, fn, noop, function () {
            if (!this.stopped()) {
              var bucket = (event.indexOf('*') === -1 ? listener : matcher);
              bucket(event)[type].push(fn);
            }
          });
        });
      }

      var vars = "a,b,c,d,e,f,g,h,i,j,k,l";
      function registerOnce(type) {
        return multiple(function (event, fn) {
          /*jslint evil:true*/
          var self = this;
          var proxy;
          if (fn.length) {
            eval('proxy = (function (' + vars.substring(0, fn.length * 2 - 1) +
              ') { self.un(event, proxy); return fn.apply(this, arguments); });');
          } else {
            proxy = function () {
              self.un(event, proxy);
              return fn.apply(this, arguments);
            };
          }
          this[type](event, proxy);
        });
      }

      var removeListener = multiple(function (event, fn) {
        this.emit('removeListener', event, fn, noop, function () {
          if (!this.stopped()) {
            var entry;
            if (event.indexOf('*') === -1) {
              entry = listener(event);
            } else {
              entry = matcher(event);
            }
            remove(entry.on,      fn);
            remove(entry.before,  fn);
            remove(entry.after,   fn);
          }
        });
      });

      var addListener = register('on');

      return {

        before         : register('before'),
        on             : addListener,
        addListener    : addListener,
        after          : register('after'),
        once           : registerOnce('on'),
        onceBefore     : registerOnce('before'),
        onceAfter      : registerOnce('after'),
        un             : removeListener,
        removeListener : removeListener,

        removeAllListeners: function (event) {
          if (arguments.length) {
            assertType(event, 'string', 'event');
            if (event.indexOf('*') === -1) {
              delete listeners[event];
            } else {
              clearEntry(matcher(event));
            }
          } else {
            listeners = {};
            matchers.length = 0;
          }
        },

        removeAllMatching: function (event) {
          assertType(event, 'string', 'event');
          var re = eventRE(event), evt, i, l;
          for (evt in listeners) {
            if (listeners.hasOwnProperty(evt) && re.test(evt)) {
              delete listeners[evt];
            }
          }
          for (i = 0, l = matchers.length; i < l; i++) {
            var entry = matchers[i];
            if (re.test(entry.event)) {
              clearEntry(entry);
            }
          }
        },

        listeners: function (event) {
          assertType(event, 'string', 'event');
          var entry;
          if (event.indexOf('*') === -1) {
            entry = listeners[event];
            if (!entry) {
              return [];
            }
          } else {
            entry = matcher(event);
          }
          return entryListeners(entry);
        },

        listenersMatching: function (event) {
          assertType(event, 'string', 'event');
          var matches = [], re = eventRE(event), evt, i, l;
          for (evt in listeners) {
            if (listeners.hasOwnProperty(evt) && re.test(evt)) {
              arrayPush.apply(matches, entryListeners(listeners[evt]));
            }
          }
          for (i = 0, l = matchers.length; i < l; i++) {
            var entry = matchers[i];
            if (entry.re.test(event) || re.test(entry.event)) {
              arrayPush.apply(matches, entryListeners(entry));
            }
          }
          return matches;
        },

        emit: function (event) {
          if (arguments.length === 0) {
            throw new TypeError(NOT_ENOUGH_ARGS_ERRORS[0]);
          }
          assertType(event, 'string', 'event');
          var args  = Array.prototype.slice.call(arguments, 1),
            options = DEFAULT_OPTIONS,
            callback,
            re,
            scope,
            stopped  = false,
            dotIndex = event.lastIndexOf('.');

          if (typeof args[args.length - 1] === 'function') {
            callback = args.pop();
          }
          if (args[args.length - 1] instanceof Options) {
            options = args.pop();
          }
          if (event.indexOf('*') !== -1) {
            re = eventRE(event);
          }

          if (dotIndex === -1) {
            scope = createPrototype(this);
            scope.hub = this;
          } else {
            var namespace = event.substring(0, dotIndex);
            scope = new View(this, namespace);
          }
          scope.event   = event;
          scope.args    = function () { return args.slice(); };
          scope.stop    = function () { stopped = true; };
          scope.stopped = function () { return stopped; };
          scope.options = options;

          function emitPhase(finder, type, args, values, then, after) {
            var listeners = finder(type, event, re);
            emit(listeners, scope, args, values, function (err, values) {
              if (err && after) {
                after(err);
              } else if (stopped) {
                done(scope, callback, err, result(scope, values));
              } else {
                then(err, values);
              }
            });
          }

          function after(err, values) {
            var res   = result(scope, values);
            var args  = [err, res];
            var empty = [];
            emitPhase(findListeners, 'after', args, empty, function () {
              emitPhase(findMatchers, 'after', args, empty, function () {
                done(scope, callback, err, res);
              });
            });
          }

          emitPhase(findMatchers, 'before', args, [], function (e, values) {
            emitPhase(findListeners, 'before', args, values,
              function (e, values) {
                emitPhase(findMatchers, 'on', args, values, function (e, values) {
                  emitPhase(findListeners, 'on', args, values, after);
                }, after);
              }, after);
          }, after);

        },

        view: function (namespace) {
          assertType(namespace, 'string', 'namespace');
          return new View(this, namespace);
        }

      };
    }


    var hub = function (listeners) {
      var instance = create();
      if (listeners) {
        instance.on(listeners);
      }
      return instance;
    };

    hub.listen  = listen;
    hub.View    = View;
    hub.Options = Options;
    hub.options = function (config) {
      return new hub.Options(config);
    };

    module.exports = hub;

  },
  'node_modules/hubjs/node_modules/listen/lib/listen': function (module, exports, require) {
    /**
     * listen.js
     *
     * Copyright (c) 2012-2013 Maximilian Antoni <mail@maxantoni.de>
     *
     * @license MIT
     */
    'use strict';
    var ErrorList    = require('node_modules/hubjs/node_modules/listen/lib/error-list');
    var TimeoutError = require('node_modules/hubjs/node_modules/listen/lib/timeout-error');

    function listen(initialValues) {
      var values;
      if (arguments.length) {
        if (Object.prototype.toString.call(initialValues) !== '[object Array]') {
          throw new TypeError('Array expected');
        }
        values = Array.prototype.slice.call(initialValues);
      } else {
        values = [];
      }

      var offset = values.length;
      var count  = 0;
      var called = 0;
      var errList;
      var handler;

      function resolve(fn) {
        if (errList) {
          fn(errList.length === 1 ? errList[0] : new ErrorList(errList));
        } else {
          fn(null, values);
        }
      }

      function assertUnresolved() {
        if (handler) {
          throw new Error("Cannot be called after then");
        }
      }

      function pushError(e) {
        if (!errList) {
          errList = [];
        }
        if (e.name === 'ErrorList') {
          Array.prototype.push.apply(errList, e.errors);
        } else {
          errList.push(e);
        }
      }

      function withTimeout(delegate, timeout) {
        var timer = setTimeout(function () {
          delegate(new TimeoutError());
          delegate = null;
        }, timeout);
        return function (err, value) {
          if (delegate) {
            clearTimeout(timer);
            delegate(err, value);
          }
        };
      }

      function listener(fn, timeout) {
        if (typeof fn === 'number') {
          timeout = fn;
          fn      = null;
        }
        assertUnresolved();
        var index = offset + count++;
        var callback = function (err, value) {
          if (fn) {
            try {
              fn(err, value);
            } catch (e) {
              if (e !== err) {
                pushError(e);
              }
            }
          }
          if (err) {
            pushError(err);
          }
          if (value !== undefined) {
            values[index] = value;
          }
          if (++called === count && handler) {
            resolve(handler);
          }
        };
        return timeout ? withTimeout(callback, timeout) : callback;
      }

      listener.push = function push(value) {
        assertUnresolved();
        values[offset + count] = value;
        offset++;
      };

      listener.err = function err(e) {
        assertUnresolved();
        pushError(e);
      };

      listener.then = function then(fn) {
        if (handler) {
          throw new Error('Cannot be called more than once');
        }
        if (typeof fn !== 'function') {
          throw new TypeError('Function expected');
        }
        handler = fn;
        if (called === count) {
          resolve(fn);
        }
      };

      return listener;
    }

    listen.ErrorList = ErrorList;

    module.exports = listen;

  },
  'node_modules/hubjs/node_modules/listen/lib/error-list': function (module, exports, require) {
    /**
     * listen.js
     *
     * Copyright (c) 2012 Maximilian Antoni <mail@maxantoni.de>
     *
     * @license MIT
     */
    'use strict';
    function ErrorList(errors) {
      this.name     = 'ErrorList';
      var sep       = '\n  - ';
      this.message  = 'Multiple callbacks err\'d:' + sep + errors.join(sep);
      this.errors   = errors;
    }

    ErrorList.prototype = Error.prototype;

    module.exports = ErrorList;

  },
  'node_modules/hubjs/node_modules/listen/lib/timeout-error': function (module, exports, require) {
    /**
     * listen.js
     *
     * Copyright (c) 2012 Maximilian Antoni <mail@maxantoni.de>
     *
     * @license MIT
     */
    'use strict';
    function TimeoutError() {
      this.name = 'TimeoutError';
    }

    TimeoutError.prototype = Error.prototype;

    module.exports = TimeoutError;

  }
}));
